#include "Deer/Scene.h"

#include "Deer/ComponentScript.h"
#include "Deer/Components.h"
#include "Deer/Enviroment.h"
#include "Deer/Log.h"
#include "Deer/Memory.h"
#include "Deer/ScriptEngine.h"
#include "Deer/VoxelWorld.h"
#include "Deer/Voxels/Chunk.h"
#include "Deer/Voxels/Layer.h"
#include "Deer/Voxels/VoxelWorldData.h"

#include "Deer/Enviroment.h"

#include "Deer/Scene/SceneData.h"

#ifdef DEER_RENDER
#include "DeerRender/Voxels/VoxelWorldRenderData.h"
#include "DeerRender/Mesh.h"
#endif

namespace Deer {
	void Scene::clear() {
		environment.clear();
		VoxelWorld::clear();
#ifdef DEER_RENDER
		MeshManager::unloadAllModels();
#endif
	}

	bool Scene::getExecutingState() {
		return isExecuting;
	}

	void Scene::beginExecution() {
		DEER_CORE_ASSERT(!isExecuting, "Deer scene is already executing");
		isExecuting = true;

		DEER_CORE_INFO("Executing Scene...");
		ScriptEngine::beginExecutionContext();

		// Instantiate all the scripts
		auto view = environment.m_registry.view<ScriptComponent, TagComponent>();
		for (auto& entID : view) {
			auto& tagComponent = view.get<TagComponent>(entID);
			auto& componentScript = view.get<ScriptComponent>(entID);

			Entity& entity = environment.getEntity(tagComponent.entityUID);
			componentScript.roeInstance = 
				ScriptEngine::createComponentScriptInstance(componentScript.scriptID, entity);

			componentScript.roeInstance->start();
		}
	}

	void Scene::tick() {
		// Update all scripts
		auto view = environment.m_registry.view<ScriptComponent>();
		for (auto& entID : view) {
			auto& componentScript = view.get<ScriptComponent>(entID);
			componentScript.roeInstance->tick();
		}
	}

	void Scene::endExecution() {
		DEER_CORE_ASSERT(isExecuting, "Deer scene is not executing");
		isExecuting = false;

		// Deatach all scripts
		auto view = environment.m_registry.view<ScriptComponent>();
		for (auto& entID : view) {
			auto& componentScript = view.get<ScriptComponent>(entID);
			componentScript.roeInstance.reset();
		}

		DEER_CORE_INFO("Stoping Scene...");
	}

}  // namespace Deer
