#include "DeerRender/Mesh.h"
#include "Deer/Log.h"

namespace Deer {
	namespace MeshManager {
        size_t minModelId = 0;
        VertexArray* meshes[SCENE_MAX_MESH_COUNT]{};
		VertexArray* loadModelToGPU(const MeshData& data);
	}

	void MeshManager::unloadAllModels() {
		for (uint16_t i = 0; i < minModelId; i++) {
			delete meshes[i];
			meshes[i] = nullptr;
		}

		minModelId = 0;
	}

	uint16_t MeshManager::loadModel(const MeshData& meshData) {
		if (minModelId >= SCENE_MAX_MESH_COUNT) {
			DEER_CORE_ERROR("Max model loaded into a scene");
			return -1;
		}

		uint16_t modelId = minModelId;
		minModelId++;

		meshes[modelId] = loadModelToGPU(meshData);

		return modelId;
	}

	VertexArray& MeshManager::getModel(uint16_t model_id) {
		DEER_CORE_ASSERT(model_id >= 0 && model_id < minModelId, "Invalid model id, id is not loaded");

		return *meshes[model_id];
	}

    VertexArray* MeshManager::loadModelToGPU(const MeshData& data) {
        VertexArray* vertexArray = VertexArray::create();
        vertexArray->bind();

        Ref<VertexBuffer> vertexPositionBuffer = VertexBuffer::create(data.vertexPositionsData, data.vertexCount * sizeof(VertexPosition));

        BufferLayout positionLayout({
            {"v_position", DataType::Short3, ShaderDataType::FloatingPoint}
        }, sizeof(VertexPosition)); // 
        BufferLayout normalLayout({
            {"v_normal", DataType::Byte3, ShaderDataType::NormalizedFloatingPoint}
        }, sizeof(VertexNormal));

        vertexPositionBuffer->bind();
        vertexPositionBuffer->setLayout(positionLayout);
        vertexArray->addVertexBuffer(vertexPositionBuffer);

        Ref<VertexBuffer> vertexNormalBuffer = VertexBuffer::create(data.vertexNormalData, data.vertexCount * sizeof(VertexNormal));
        vertexNormalBuffer->bind();
        vertexNormalBuffer->setLayout(normalLayout);
        vertexArray->addVertexBuffer(vertexNormalBuffer);

        Ref<IndexBuffer> indexBuffer = IndexBuffer::create(data.indexData, data.indexCount * sizeof(uint16_t), IndexDataType::Unsigned_Short);
        indexBuffer->bind();
        vertexArray->setIndexBuffer(indexBuffer);

        return vertexArray;
    }
}