#include "DeerRender/Shader.h"
#include "Deer/Log.h"

namespace Deer {
	namespace ShaderManager {
        size_t minShaderId = 0;
        Shader* shaders[SCENE_MAX_SHADER_COUNT]{};
	}

	void ShaderManager::unloadAllShaders() {
		for (uint16_t i = 0; i < minShaderId; i++) {
			delete shaders[i];
			shaders[i] = nullptr;
		}

		minShaderId = 0;
	}

	uint16_t ShaderManager::loadShader(const ShaderData& data) {
		if (minShaderId >= SCENE_MAX_SHADER_COUNT) {
			DEER_CORE_ERROR("Max shader loaded into a scene");
			return -1;
		}

		uint16_t shaderId = minShaderId;
		minShaderId++;

		shaders[shaderId] = Shader::create(data.vertexShader, data.fragmentShader);

		return shaderId;
	}

	Shader& ShaderManager::getShader(uint16_t shaderId) {
		DEER_CORE_ASSERT(shaderId >= 0 && shaderId < minShaderId, "Invalid shader id, id is not loaded");

		return *shaders[shaderId];
	}
}