#include "DeerStudio/EditorEngine.h"
#include "DeerStudio/EditorEngine/ErrorHandle.h"
#include "DeerStudio/EditorEngine/API.h"

#include "scripthandle.h"
#include "scriptany.h"
#include "angelscript.h"

namespace Deer {
    namespace EditorEngine { 

        void registerEntityStructs() {
            AS_CHECK(scriptEngine->RegisterObjectType("Entity", sizeof(EntityStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityStruct>() | asOBJ_APP_CLASS_ALLINTS));

            AS_CHECK(scriptEngine->RegisterObjectBehaviour(
                "Entity", asBEHAVE_CONSTRUCT, "void f(int)",
                asFunctionPtr(constructEntityStruct),
                asCALL_CDECL_OBJLAST
            ));

            AS_CHECK(scriptEngine->RegisterObjectType("EntityChilds", sizeof(EntityStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityStruct>() | asOBJ_APP_CLASS_ALLINTS));

            AS_CHECK(scriptEngine->RegisterObjectType("TransformComponent", sizeof(EntityStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityStruct>() | asOBJ_APP_CLASS_ALLINTS));

        }

        void registerEntityFunctions() {
            // ENTITY
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "string get_name() const property",
                asMETHOD(EntityStruct, getName),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void set_name(string& in) property",
                asMETHOD(EntityStruct, setName),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "int get_id() const property",
                asMETHOD(EntityStruct, getId),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "Entity createChild(const string& in)",
                asMETHOD(EntityStruct, createChild),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool get_isRoot() const property",
                asMETHOD(EntityStruct, isRoot),
                asCALL_THISCALL
            ));
            
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void destroy()",
                asMETHOD(EntityStruct, destroy),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool get_exists() const property",
                asMETHOD(EntityStruct, exists),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "Entity get_parent() property",
                asMETHOD(EntityStruct, getParent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void set_parent(Entity) property",
                asMETHOD(EntityStruct, setParent),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool isDescendantOf(Entity)",
                asMETHOD(EntityStruct, isDescendantOf),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool opEquals(const Entity &in) const",
                asMETHOD(EntityStruct, opEquals),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "EntityChilds get_childs() const property",
                asMETHOD(EntityStruct, getSelf),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "TransformComponent get_transform() const property",
                asMETHOD(EntityStruct, getSelf),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterGlobalFunction(
                "Entity getRoot()",
                asFUNCTION(getRoot),
                asCALL_CDECL
            ));
        
            // ENTITY CHILD
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "EntityChilds",
                "int get_count() const property",
                asMETHOD(EntityChildArrayStruct, getChildCount),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "EntityChilds",
                "Entity opIndex(int) const",
                asMETHOD(EntityChildArrayStruct, getChild),
                asCALL_THISCALL
            ));

            // TRANSFORM COMPONENT
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "vec3 get_position() const property",
                asMETHOD(TransformComponentStruct, getPosition),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "vec3 get_scale() const property",
                asMETHOD(TransformComponentStruct, getScale),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "vec3 get_rotation() const property",
                asMETHOD(TransformComponentStruct, getRotation),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "void set_position(const vec3) property",
                asMETHOD(TransformComponentStruct, setPosition),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "void set_scale(const vec3) property",
                asMETHOD(TransformComponentStruct, setScale),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "void set_rotation(const vec3) property",
                asMETHOD(TransformComponentStruct, setRotation),
                asCALL_THISCALL
            ));
        }

        void registerTransformComponent() {
            AS_CHECK(scriptEngine->RegisterObjectType("Entity", sizeof(EntityStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityStruct>() | asOBJ_APP_CLASS_ALLINTS));


            AS_CHECK(scriptEngine->RegisterObjectBehaviour(
                "Entity", asBEHAVE_CONSTRUCT, "void f(int)",
                asFunctionPtr(constructEntityStruct),
                asCALL_CDECL_OBJLAST
            ));
        }
    }   
}   