#include "DeerStudio/EditorEngine.h"
#include "DeerStudio/EditorEngine/ErrorHandle.h"
#include "DeerStudio/EditorEngine/API.h"

#include "scripthandle.h"
#include "scriptany.h"
#include "angelscript.h"
#include "glm/glm.hpp"

namespace Deer {
    namespace EditorEngine { 
        void registerMathStructs() {
            AS_CHECK(scriptEngine->RegisterObjectType(
                "vec3",
                sizeof(glm::vec3),
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<glm::vec3>() | asOBJ_APP_CLASS_ALLFLOATS));
            
            AS_CHECK(scriptEngine->RegisterObjectBehaviour("vec3", asBEHAVE_CONSTRUCT,
                "void f()", asFUNCTION(vec3_constructor), asCALL_CDECL_OBJLAST));
            
            AS_CHECK(scriptEngine->RegisterObjectBehaviour("vec3", asBEHAVE_CONSTRUCT,
                "void f(float, float = 0, float = 0)", asFUNCTION(vec3_constructor_params), asCALL_CDECL_OBJLAST));
                
            AS_CHECK(scriptEngine->RegisterObjectProperty("vec3", "float x", asOFFSET(glm::vec3, x)));
            AS_CHECK(scriptEngine->RegisterObjectProperty("vec3", "float y", asOFFSET(glm::vec3, y)));
            AS_CHECK(scriptEngine->RegisterObjectProperty("vec3", "float z", asOFFSET(glm::vec3, z)));
                
        }

        void registerMathFunctions() {
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "vec3",
                "vec3 opAdds(const vec3 &in)",
                asFUNCTION(vec3_add),
                asCALL_CDECL_OBJLAST
            ));
            
            AS_CHECK(scriptEngine->RegisterObjectMethod("vec3",
                "vec3 opSub(const vec3 &in) const",
                asFUNCTION(vec3_sub),
                asCALL_CDECL_OBJLAST
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod("vec3",
                "vec3 opNeg() const",
                asFUNCTION(vec3_neg),
                asCALL_CDECL_OBJLAST
            )); 
            
            AS_CHECK(scriptEngine->RegisterObjectMethod("vec3",
                "vec3 opMul(float) const",
                asFUNCTION(vec3_mult),
                asCALL_CDECL_OBJLAST
            ));

        }
    }   
}   